/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import javax.servlet.jsp.PageContext;

import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.NodeMatcher;

/**
 * <p>
 * Helper class for the Bread Crumb Trail Tag Component.
 * </p>
 *
 * <p>
 * Note, since this is a multi-package component, this helper class is defined as public instead of package-private
 * to avoid defining such class in every package. This is allowed by the TopCoder rule.
 * All the static methods are shared between multiple packages are defined as public, otherwise will be defined as
 * package-private.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 *
 */
public final class Util {
    /**
     * <p>
     * This private constructor prevents to create a new instance.
     * </p>
     */
    private Util() {
    }

    /**
     * <p>
     * Checks whether the given Object is null.
     * </p>
     *
     * @param arg the argument to check
     * @param name the name of the argument to check
     *
     * @throws IllegalArgumentException if the given Object is null
     * @return the original object
     */
    public static Object checkNull(Object arg, String name) {
        if (arg == null) {
            throw new IllegalArgumentException(name + " should not be null.");
        }

        return arg;
    }

    /**
     * <p>
     * Checks whether the given String is null or empty.
     * Here empty means the length of the given string is zero after trimmed.
     * </p>
     *
     * @param arg the String to check
     * @param name the name of the String argument to check
     *
     * @throws IllegalArgumentException if the given string is null or empty
     * @return the original string
     */
    public static String checkString(String arg, String name) {
        checkNull(arg, name);

        if (arg.trim().length() == 0) {
            throw new IllegalArgumentException(name + " should not be empty.");
        }

        return arg;
    }

    /**
     * <p>
     * Returns true if the given argument is empty string(trimmed), otherwise return false.
     * </p>
     *
     * @param arg the string argument to judge whether it's empty or not
     *
     * @return true if the given argument is empty string(trimmed), otherwise return false.
     */
    public static boolean isEmptyString(String arg) {
        return (arg != null && arg.trim().length() == 0);
    }

    /**
     * <p>
     * Returns the value associated with the given attribute name under page context.
     * </p>
     *
     * <p>
     * Note, in the following two cases BreadCrumbException will be thrown:
     * <ol>
     * <li>The given attribute is not found or null in page context and canNull is false</li>
     * <li>The type of the attribute value is not of the type specified by expectedClass argument</li>
     * </ol>
     * </p>
     *
     * @param pageContext page context to access the given attribute
     * @param attrName the attribute name in page context
     * @param scope scope value for PageContext class
     * @param expectedClass the value associated with the given attribute name should be of this type
     * @param canNull flag to identify that the value can be null or null
     *
     * @return the value associated with the given attribute name under page context.
     *
     * @throws BreadCrumbException the given attribute is not found or null in page context and canNull is false, or
     * the type of the attribute value is not of the type specified by expectedClass argument
     */
    public static Object getPageContextAttribute(PageContext pageContext, String attrName, int scope,
            Class expectedClass, boolean canNull) throws BreadCrumbException {
        Object obj = pageContext.getAttribute(attrName, scope);

        //The given attribute is not found or null in page context and canNull is false
        if (obj == null && !canNull) {
            throw new BreadCrumbException("Attribute " + attrName + " in pageContext is null.");
        }

        //The type of the attribute value is not of the type specified by expectedClass argument
        if (obj != null && !expectedClass.isAssignableFrom(obj.getClass())) {
            throw new BreadCrumbException("Attribute " + attrName + " in pageContext is not of "
                    + expectedClass.getName() + " type");
        }

        return obj;
    }

    /**
     * <p>
     * Searches the siteMap to found all the shortest path from all the root nodes to all the nodes that match
     * the given BreadCumbNode instance.
     * </p>
     *
     * <p>
     * In this method, Breadth-First-Search is used to iterator every nodes in siteMap and updates the shortest path
     * when processing each node.
     * </p>
     *
     * @param siteMap the map of the site, the key is NodeMatcher instance and the value is a list and stores children
     * of the key element.
     * @param node the current node to search the shortest path from root nodes to all the nodes(of NodeMathcer type)
     * that match this given node(of BreadCrumbNode type).
     *
     * @return all the shortest path in siteMap from root nodes to all the nodes(of NodeMathcer type) that match the
     * given node (of BreadCrumbNode type), if no shortest path is found, then null will be returned.
     *
     */
    public static List shortestPath(Map siteMap, BreadCrumbNode node) {
        checkNull(siteMap, "siteMap");
        checkNull(node, "node");

        // a map instance stores useful information for searching, the key is NodeMatcher instance while the value is
        // Data instance, please refer to Data documentation for details
        Map dataMap = new HashMap();

        // the queue using in Breadth-First-Search
        // All the root nodes will be added to the queue before searching
        LinkedList queue = new LinkedList();

        // the terminal nodes (which matches the current node), the element is of NodeMatcher type
        Set terminalNodes = new HashSet();

        initialize(siteMap, node, dataMap, queue, terminalNodes);

        // if no root nodes or no NodeMatcher instance matches the given node, then null is returned
        // to identify no shortest path is found
        if (queue.isEmpty() || terminalNodes.isEmpty()) {
            return null;
        }

        // stores the visited NodeMatcher instances while Breadth-First-Search
        Set visitedNodes = new HashSet();

        // the end condition is the queue is empty
        while (!queue.isEmpty()) {
            // mark the current NodeMatcher instance visited and remove it from the queue
            NodeMatcher curNode = (NodeMatcher) queue.removeFirst();
            visitedNodes.add(curNode);

            Data curData = (Data) dataMap.get(curNode);

            List children = (List) siteMap.get(curNode);

            // iterate the children of current node to update the shortest path data
            for (Iterator it = children.iterator(); it.hasNext();) {
                NodeMatcher childNode = (NodeMatcher) it.next();

                Data childData = (Data) dataMap.get(childNode);

                if (childData.getLength() > curData.getLength() + 1) {
                    // find a shorter path, the old path will be clear
                    // new path will be added in the next step
                    childData.clearPathList();
                    // the length is set first to make sure the next step will be executed
                    childData.setLength(curData.getLength() + 1);
                }

                // an equal length of path is found, add it to the pathList
                if (childData.getLength() == curData.getLength() + 1) {
                    for (Iterator sIt = curData.getPathList().iterator(); sIt.hasNext();) {
                        List path = (List) sIt.next();
                        List newPath = new ArrayList(path);
                        newPath.add(childNode);
                        childData.addPath(newPath);
                    }
                }

                // a child node will be added to queue in case that it is not in queue or in visitedNodes
                if (!visitedNodes.contains(childNode) && !queue.contains(childNode)) {
                    queue.addLast(childNode);
                }
            }
        }

        return shortestPath(dataMap, terminalNodes);
    }

    /**
     * <p>
     * Returns the shortest path list in all terminal nodes.
     * </p>
     *
     * <p>
     * Note, the data in dataMap should be set up already before calling this method, which means the search has been
     * done to the site map and shortest path list for each node is calculated already.
     * </p>
     *
     * @param dataMap a map instance stores useful information for searching, the key is NodeMatcher instance while the
     * value is Data instance, please refer to Data documentation for details.
     * @param terminalNodes the terminal nodes (which matches the current node)
     *
     * @return the shortest path list in all terminal nodes.
     */
    private static List shortestPath(Map dataMap, Set terminalNodes) {
        // stores the final shortest path list in all the terminal nodes
        List pathList = new ArrayList();

        // stores the currently found shortest length
        int minLength = Integer.MAX_VALUE - 1;

        // iterates all the NodeMatcher nodes that match the given BreadCrumbNode node
        // then the final shortest path will be returned
        for (Iterator it = terminalNodes.iterator(); it.hasNext();) {
            Data destData = (Data) dataMap.get(it.next());
            if (destData.length < minLength) {
                pathList.clear();
                minLength = destData.length;
            }
            if (destData.length <= minLength) {
                pathList.addAll(destData.getPathList());
            }
        }

        return pathList;
    }

    /**
     * <p>
     * Initializes all the data needed by the search of shortest path in the site map graph.
     * </p>
     *
     * <p>
     * Note, the data to initialized includes:
     * <ol>
     * <li>map - of Map type, the key is NodeMathcer instance, while the value is Data instance, stores useful
     * information in the search, please refer to Data documentation for details.</li>
     * <li>roots - of List type, the element is NodeMatcher instance, represents the root nodes in site map.</li>
     * <li>terminalNodes - of NodeMatcher type, the element is NodeMatcher instance, represents the terminal nodes
     * (which matches the current node).</li>
     * </ol>
     * </p>
     *
     * @param siteMap the map of the site, the key is NodeMatcher instance and the value is a list and stores children
     * of the key element.
     * @param node the current node
     * @param dataMap a map instance stores useful information for searching, the key is NodeMatcher instance while the
     * value is Data instance, please refer to Data documentation for details
     * @param roots the root nodes in site map
     * @param terminalNodes the terminal nodes (which matches the current node)
     */
    private static void initialize(Map siteMap, BreadCrumbNode node, Map dataMap, List roots, Set terminalNodes) {
        // all the NodeMatcher instances in siteMap will be stored in dataMap.
        // the terminalNodes will be initialized too.
        for (Iterator it = siteMap.keySet().iterator(); it.hasNext();) {
            NodeMatcher curNode = (NodeMatcher) it.next();
            if (curNode.matches(node)) {
                terminalNodes.add(curNode);
            }
            dataMap.put(curNode, new Data());
        }

        // set up the in-dgree information of each NodeMatcher instance
        for (Iterator it = siteMap.values().iterator(); it.hasNext();) {
            List children = (List) it.next();
            for (int i = 0; i < children.size(); i++) {
                NodeMatcher curNode = (NodeMatcher) children.get(i);
                Data data = (Data) dataMap.get(curNode);
                data.incDegree();
            }
        }

        // found out all the root nodes if it has a zero in-degree.
        for (Iterator it = dataMap.entrySet().iterator(); it.hasNext();) {
            Entry entry = (Entry) it.next();
            NodeMatcher curNode = (NodeMatcher) entry.getKey();
            Data data = (Data) entry.getValue();
            // in-degree is used to judge whether the node is a root node or not
            if (data.getDegree() == 0) {
                // the root node has a one-length shortest path, the path contains the root node itself
                roots.add(curNode);
                data.setLength(1);
                List children = new LinkedList();
                children.add(curNode);
                data.addPath(children);
            }
        }

    }

    /**
     * <p>
     * A data class to store information that is useful while searing the site map graph to find all of the
     * shortest path.
     * </p>
     *
     * <p>
     * The useful information includes :
     * <ol>
     * <li>degree - in-degree of the node, used to judge whether the node is a root node or not.</li>
     * <li>length - the length of shortest path, used to choose path when a new path is found.</li>
     * <li>pathList - the current found shortest path list.</li>
     * </ol>
     * </p>
     *
     * <p>
     * Each node(of NodeMatcher type) in site map will own a Data instance.
     * </p>
     *
     * @author TCSDEVELOPER
     * @version 2.0
     * @since 2.0
     *
     */
    private static final class Data {
        /**
         * <p>
         * Stores the in-degree of the node, used to judge whether the node is a root node or not.
         * </p>
         *
         * <p>
         * Initially it is zero, can be changed by incDegree() and retrieved by getDegree().
         * </p>
         */
        private int degree = 0;

        /**
         * <p>
         * Stores the length of shortest path, used to choose path when a new path is found.
         * </p>
         *
         * <p>
         * Initially it is max value of int to identify no path found yet, can be changed by setLength() and retrieved
         * by getPathList().
         * </p>
         */
        private int length = Integer.MAX_VALUE;

        /**
         * <p>
         * Stores the current found shortest path list.
         * </p>
         *
         * <p>
         * The element in pathList should be of List type, and element in a path is of NodeMatcher type.
         * </p>
         *
         * <p>
         * Initially it is empty to identify no path found yet, can be changed by addPath(List) and clearPath(),
         * and retrieved by getLength().
         * </p>
         */
        private List pathList = new LinkedList();

        /**
         * <p>
         * Returns the length of shortest path currently found.
         * </p>
         *
         * @return the length of shortest path
         */
        int getLength() {
            return length;
        }

        /**
         * <p>
         * Sets the new length of shortest path, it is used when a shorter path is found.
         * </p>
         *
         * @param length the new length of shortest path
         */
        void setLength(int length) {
            this.length = length;
        }

        /**
         * <p>
         * Returns the in-degree of the node.
         * </p>
         *
         * @return the in-degree of the node.
         */
        int getDegree() {
            return degree;
        }

        /**
         * <p>
         * Increase the in-degree of the node by one.
         * It is used when the node is found as a child of some node.
         * </p>
         *
         */
        void incDegree() {
            degree++;
        }

        /**
         * <p>
         * Adds a new path to the pathList.
         * Note, the length of path should be the same with the length field in the node.
         * </p>
         *
         * @param path a new shortest path.
         * @see Data#pathList
         */
        void addPath(List path) {
            pathList.add(path);
        }

        /**
         * <p>
         * Clears the pathList to make the pathList empty.
         * It is used when a shorter path is found.
         * </p>
         *
         * @see Data#pathList
         */
        void clearPathList() {
            this.pathList.clear();
        }

        /**
         * <p>
         * Returns the list of shortest path currently found.
         * </p>
         *
         * @return the list of shortest path currently found.
         * @see Data#pathList
         */
        List getPathList() {
            return pathList;
        }

    }
}
