/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;
import com.topcoder.web.ui.tag.breadcrumbtrail.Util;


/**
 * <p>
 * This tag will format the node and style using a template file.
 * </p>
 *
 * <p>
 * A template file is a separate file that will be included in the out stream.
 * This class will take the current node/style and define them as request attributes when the template file is
 * included (those attributes can then be used by the template file to format the correct representation of them).
 * </p>
 *
 * <p>
 * <b>Note: </b> This node formatter tag will be used in the body of a BreadCrumbTrailTag and will format the current
 * node/style using a template file.
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> This class is thread safe in the context of the JSP rendering process.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 *
 */
public class TemplateNodeFormatterTag extends TagSupport {
    /**
     * <p>
     * Represents the template file that will be used when formatting this tag.
     * </p>
     *
     * <p>
     * This variable is set by the constructor, is mutable and can be null or an empty (trim'd) string to specify
     * template is to be used (and thus nothing produced by this tag).
     * </p>
     *
     * <p>
     * This variable is referenced by the associated getter/setter methods and the doStartTag methods.
     * </p>
     *
     * <p>
     * Note, it is expected (but not required) that the value will be a non-null, non-empty value.
     * </p>
     *
     */
    private String template;

    /**
     * <p>
     * Constructs the TemplateNodeFormatterTag using no constructor parameters.
     * </p>
     *
     */
    public TemplateNodeFormatterTag() {
        // empty
    }

    /**
     * <p>
     * Constructs the TemplateNodeFormatterTag using the configuration manager to set the initial value of the template.
     * </p>
     *
     * @param template A possibly null, possibly empty (trim'd) string representing the template to use
     */
    public TemplateNodeFormatterTag(String template) {
        this.template = template;
    }

    /**
     * <p>
     * This is the getter method for the template variable.
     * </p>
     *
     * @return A possibly null, possibly empty (trim'd) string representing the template to use
     */
    public String getTemplate() {
        return template;
    }

    /**
     * <p>
     * Sets the template that will be used for rendering this tag.
     * </p>
     *
     * @param template A possibly null, possibly empty (trim'd) string representing the template to use
     */
    public void setTemplate(String template) {
        this.template = template;
    }

    /**
     * <p>
     * This method will take the current node and format, make them into request attributes and include the
     * template file (which is assumed to use the request attributes).
     * </p>
     *
     * <p>
     * Note, If template is null or an empty (trim'd) string, this tag will not render anything.
     * </p>
     *
     * @return Always SKIP_BODY to skip body evaluation
     *
     * @throws BreadCrumbException if any exception occurs during processing
     */
    public int doStartTag() throws BreadCrumbException {
        if (template == null || template.trim().length() == 0) {
            return SKIP_BODY;
        }

        // locates the node to be formatted and the style class to be used in page context
        // if the node to be formatted can't be found, then BreadCrumbException will be thrown
        // the style class is allowed to be null or empty
        BreadCrumbNode node = (BreadCrumbNode) Util.getPageContextAttribute(pageContext,
                BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE, BreadCrumbNode.class, false);

        String style = (String) Util.getPageContextAttribute(pageContext,
                BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, PageContext.PAGE_SCOPE, String.class, true);

        // if style class is empty, it will be set to null instead
        if (Util.isEmptyString(style)) {
            style = null;
        }

        pageContext.getRequest().setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, node);
        pageContext.getRequest().setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, style);

        try {
            pageContext.include(template);
        } catch (ServletException e) {
            throw new BreadCrumbException("ServletException occurs when including template file, caused by "
                + e.getMessage(), e);
        } catch (IOException e) {
            throw new BreadCrumbException("IOException occurs when including template file, cause by "
                + e.getMessage(), e);
        }

        return SKIP_BODY;
    }
}
