/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import java.io.Serializable;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.Util;

/**
 * <p>
 * This class is a typical pair class that holds a paired association of a node matcher to the matching node.
 * </p>
 *
 * <p>
 * The nodeMatcher describes the node that matched the above node and describes the default node information.
 * The node describes the actual node that matched the nodeMatcher and contains the contextual specific information.
 * </p>
 *
 * <p>
 * <b>Thread Safety: </b> This class is thread safe by being immutable.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 *
 */
class NodePair implements Serializable {
    /**
     * <p>
     * Represents the actual node that holds the contextual information (query string, etc) that matches the node
     * matcher.
     * </p>
     *
     * <p>
     * This variable is set by the constructor, is immutable and can never be null.
     * The node will be referenced in the getNode method.
     * </p>
     *
     */
    private final BreadCrumbNode node;

    /**
     * <p>
     * Represents the node matcher that holds the matching information and the default title.
     * </p>
     *
     * <p>
     * This variable is set by the constructor, is immutable and can be null (if there is no associated nodeMatcher).
     * The path node will be referenced in the getNodeMatcher and getNode method.
     * </p>
     *
     */
    private final NodeMatcher nodeMatcher;

    /**
     * <p>
     * Constructs the path pair from the given node and nodeMatcher
     * </p>
     *
     * <p>
     * Note, the title in the given node is null and the title in the nodeMatcher is non-null, then the title in the
     * nodeMatcher will be set to the given node(a new BreadCrumbNode will be created).
     * </p>
     *
     * @param node The non-null bread crumb node that matches the path node
     * @param nodeMatcher The possibly null node matcher
     *
     * @throws IllegalArgumentException if node is null
     */
    NodePair(BreadCrumbNode node, NodeMatcher nodeMatcher) {
        Util.checkNull(node, "node");

        if (node.getTitle() == null && nodeMatcher != null) {
            this.node = new BreadCrumbNode(node.getUrl(), nodeMatcher.getTitle());
        } else {
            this.node = node;
        }
        this.nodeMatcher = nodeMatcher;
    }

    /**
     * <p>
     * The getter method for the node.
     * </p>
     *
     * @return A non-null node
     */
    public BreadCrumbNode getNode() {
        return node;
    }

    /**
     * <p>
     * The getter method for the node matcher.
     * </p>
     *
     * @return A possibly null node matcher
     */
    public NodeMatcher getNodeMatcher() {
        return nodeMatcher;
    }
}
